#!/usr/bin/env python
"""
_ListRunsWorkflow_

MySQL implementation of DBS3Buffer.ListRunsWorkflow
"""

from WMComponent.DBS3Buffer.Oracle.ListRunsWorkflow import ListRunsWorkflow as OracleListRunsWorkflow

class ListRunsWorkflow(OracleListRunsWorkflow):
    """
    _ListRunsWorkflow_

    Identical to Oracle version
    """
