#!/usr/bin/env python
"""
_ListClosable_

Oracle implementation of Fileset.ListClosable
"""

from WMCore.WMBS.MySQL.Fileset.ListClosable import ListClosable as ListFilesetClosableMySQL

class ListClosable(ListFilesetClosableMySQL):
    pass
