#!/usr/bin/env python
"""
_GetMigratedBlocks_

Oracle implementation on PhEDExInjector.GetMigratedBlocks.
"""
from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.GetMigratedBlocks import GetMigratedBlocks as MySQLBase


class GetMigratedBlocks(MySQLBase):
    """
    _GetMigratedBlocks_

    """
    pass
