import logging

class TestRegistryFile2:


    def doSomething(self, parameters = {}):
        logging.debug("TestCollection_2.TestRegistryFile2 is doing something")
        logging.debug("With parameters: "+str(parameters))
        return parameters
