# -*- coding: utf-8 -*-
"""
_GetPendingSlots_

Oracle implementation of Locations.GetPendingSlots

Created on Mon Jun 18 13:51:29 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.GetPendingSlots import GetPendingSlots as MySQLGetPendingSlots

class GetPendingSlots(MySQLGetPendingSlots):
    """
    Same as MySQL version

    """
    pass
