#!/usr/bin/env python

from __future__ import division

from WMCore.WMBS.MySQL.Subscriptions.GetSubsWithoutJobGroup import GetSubsWithoutJobGroup as GetSubsWithoutJobGroupMySQL


class GetSubsWithoutJobGroup(GetSubsWithoutJobGroupMySQL):
    """ Same implementation as in MySQL"""
    pass
