.main-font {
    font-size: 12px;
}
.main-menu-item {
    font-size: 16px;
    color: #7C81B2;
}
.header {
    background-color: #CCCCCC;
    padding-left: 20px;
    padding-right: 20px;
    padding-top: 10px;
    border-bottom: 1px solid #000000;
    font: "Lucida Grande","Lucida Sans Unicode","Lucida Sans",Helvetica,Arial,sans-serif;
    font-weight: bold;
    color: #808080;
}
.shadow {
    -webkit-box-shadow: 10px 10px 20px #666666;
    -moz-box-shadow: 10px 10px 20px #666666;
    -o-box-shadow: 10px 10px 20px #666666;
    box-shadow: 10px 10px 20px #666666;
}
.all-side-shadow {
    -webkit-box-shadow: 2px 2px 10px 10px #666;
    -moz-box-shadow: 2px 2px 10px 10px #666;
    -o-box-shadow: 2px 2px 10px 10px #666;
    box-shadow: 2px 2px 10px 10px #666;
}
.borders {
    -webkit-border-radius: 40px;
    -khtml-border-radius: 40px;
    -moz-border-radius: 40px;
    border-radius: 40px;
}

.editor {
    width:100%;
    height:400px;
    font-family: 'Monaco', 'Menlo', 'Ubuntu Mono', 'Consolas', 'source-code-pro', monospace;
    font-weight: bold;
    font-size: 1.2em;
    color: rgb(64, 64, 64);
    position: relative;
/*    overflow: hidden;*/
/*    resize: none;*/
/*    white-space: nowrap;*/
}
.height-200 {
    height:200px;
}
.width-200 {
    width: 200px;
}

// http://css-tricks.com/snippets/css/glowing-blue-input-highlights/
input[type=text], textarea {
    -webkit-appearance: textarea;
    -webkit-transition: all 0.30s ease-in-out;
    -moz-transition: all 0.30s ease-in-out;
    -ms-transition: all 0.30s ease-in-out;
    -o-transition: all 0.30s ease-in-out;
    outline: none;
    padding: 3px 0px 3px 3px;
    margin: 5px 1px 3px 0px;
    border: 1px solid #DDDDDD;
}

input[type=text]:focus, textarea:focus {
    box-shadow: 0 0 5px #333333;
    border: 1px solid #808080;
    outline-color: #333333;
    background-color: #F5F5F5;
}
.underline {border-bottom: 3px solid #808080;}
.hide {
    display: none;
}
.show {
    display: block;
    margin-top:6px;
    border-top-width:1px;
}
.replace {
    color: blue;
    font-weight: bold;
}
.ticket {
    color: #000000;
    background-color: #F5C669;
    border-width: 1px 1px 1px 1px;
    padding: 3px 3px 3px 3px;
    border-style: dashed;
    border-color: #000;
    cursor: pointer;
}
.record {
    font-weight: bold;
    padding: 3px 3px 3px 3px;
    cursor: pointer;
}
.confirmation {
    font-weight: bold;
    position: fixed;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    z-index:999;
}
.tasks {
    background-color: #E7E5C1;
    padding-left: 5px;
    padding-right: 5px;
    padding-top: 5px;
    border: 1px solid #000000;
    width:80%;
    height:500px;
    font-weight: bold;
    position: fixed;
    top: 40%;
    left: 40%;
    transform: translate(-40%, -40%);
    z-index:999;
    overflow: scroll;
}
.taskParams {
    background-color: #E6E6E6;
    padding: 10px;
}
.alignRight {
    float: right;
    width: 100px;
    padding: 10px;
}
.fadeout {
    opacity: 0;
    -webkit-transition: opacity 4s ease-in;
    -moz-transition: opacity 4s ease-in;
    -o-transition: opacity 4s ease-in;
    -ms-transition: opacity 4s ease-in;
    transition: opacity 4s ease-in;
}
::-webkit-input-placeholder {
    color: red;
    font-weight: bold;
}
:-moz-placeholder { /* Firefox 18- */
    color: red;
    font-weight: bold;
}
::-moz-placeholder {  /* Firefox 19+ */
    color: red;
    font-weight: bold;
}
:-ms-input-placeholder {
    color: red;
    font-weight: bold;
}
tr:nth-child(even){ background:#F4F4F4; }
th { background: #B3B3B3; color: white; font-weight: bold; }

#container ul {
    list-style-type: none;
    padding: 0;
    margin: 0;
}
.lift {
    position: relative;
    top: -40px;
    bottom: -40px;
}
.visible {
    padding: 15px;
    border: solid 10px #408000;
    box-shadow: inset 5px 1px 20px 0px #ACBCD8;
    transition: box-shadow 0.3s;
}
.visible_input {
    padding: 0.4em 0.25em;
    width: 100%;
    background: transparent;
    color: red;
    font-weight: bold;
}
.invisible {
    background-color: #dddddd;
}
.invisible_input {
    width: 100%;
    color: #CCCCCC;
    box-shadow: 0 0 5px #333333;
    outline-color: #333333;
    background-color: #F5F5F5;
}
