#!/usr/bin/env python
"""
_IDFromFilesetWorkflow_

Oracle implementation of Subscriptions.IDFromFilesetWorkflow
"""




from WMCore.WMBS.MySQL.Subscriptions.IDFromFilesetWorkflow import IDFromFilesetWorkflow as IDFromFilesetWorkflowMySQL

class IDFromFilesetWorkflow(IDFromFilesetWorkflowMySQL):
    pass
