#!/usr/bin/env python
"""
_GetUnsubscribedBlocks_

Oracle implementation of GetUnsubscribedBlocks.
"""

from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.GetUnsubscribedBlocks import GetUnsubscribedBlocks as MySQLBase


class GetUnsubscribedBlocks(MySQLBase):
    pass
