#!/usr/bin/env python
"""
_ListJobsBySub_

Oracle implementation of Monitoring.ListJobsBySub
"""




from WMCore.WMBS.MySQL.Monitoring.ListJobsBySub import ListJobsBySub \
 as ListJobsBySubMySQL

class ListJobsBySub(ListJobsBySubMySQL):
    pass
