#!/usr/bin/env python
"""
_AssociateWorkflowToFile_

Oracle implementation of DBS3Buffer.DBSBufferFiles.AssociateWorkflowToFile

Created on Oct 12, 2012

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.AssociateWorkflowToFile import \
     AssociateWorkflowToFile as MySQLAssociateWorkfloToFile

class AssociateWorkflowToFile(MySQLAssociateWorkfloToFile):
    pass
